package ShareStuff::AppUtils;
# vim:noet:ts=4
use strict; use warnings;
use OpenSRF::Application;
use base qw/OpenSRF::Application/;
use OpenSRF::Utils::Cache;
use OpenSRF::Utils::Logger qw/$logger/;
use OpenSRF::EX qw(:try);
use ShareStuff::Event;
use ShareStuff::Const qw/:const/;
use Data::Dumper;
use ShareStuff::CStoreEditor;
use Unicode::Normalize;
use OpenSRF::Utils::SettingsClient;
use UUID::Tiny;
use Encode;
use DateTime;
use DateTime::Format::ISO8601;

# ---------------------------------------------------------------------------
# Pile of utilty methods used accross applications.
# ---------------------------------------------------------------------------
my $cache_client = "OpenSRF::Utils::Cache";


# ---------------------------------------------------------------------------
# on sucess, returns the created session, on failure throws ERROR exception
# ---------------------------------------------------------------------------
sub start_db_session {

	my $self = shift;
	my $session = OpenSRF::AppSession->connect( "sharestuff.cstore" );
	my $trans_req = $session->request( "sharestuff.cstore.transaction.begin" );

	my $trans_resp = $trans_req->recv();
	if(ref($trans_resp) and UNIVERSAL::isa($trans_resp,"Error")) { throw $trans_resp; }
	if( ! $trans_resp->content() ) {
		throw OpenSRF::ERROR 
			("Unable to Begin Transaction with database" );
	}
	$trans_req->finish();

	$logger->debug("Setting global cstore session to ".
		"session: " . $session->session_id . " : " . $session->app );

	return $session;
}

my $PERM_QUERY = {
    select => {
        au => [ {
            transform => 'permission.usr_has_perm',
            alias => 'has_perm',
            column => 'id',
            params => []
        } ]
    },
    from => 'au',
    where => {},
};


# returns undef if user has all of the perms provided
# returns the first failed perm on failure
sub check_user_perms {
	my($self, $user_id, $org_id, @perm_types ) = @_;
	$logger->debug("Checking perms with user : $user_id , org: $org_id, @perm_types");

	for my $type (@perm_types) {
	    $PERM_QUERY->{select}->{au}->[0]->{params} = [$type, $org_id];
		$PERM_QUERY->{where}->{id} = $user_id;
		return $type unless $self->is_true(ShareStuff::CStoreEditor->new->json_query($PERM_QUERY)->[0]->{has_perm});
	}
	return undef;
}

# checks the list of user perms.  The first one that fails returns a new
sub check_perms {
	my( $self, $user_id, $org_id, @perm_types ) = @_;
	my $t = $self->check_user_perms( $user_id, $org_id, @perm_types );
	return ShareStuff::Event->new('PERM_FAILURE', ilsperm => $t, ilspermloc => $org_id ) if $t;
	return undef;
}



# ---------------------------------------------------------------------------
# commits and destroys the session
# ---------------------------------------------------------------------------
sub commit_db_session {
	my( $self, $session ) = @_;

	my $req = $session->request( "sharestuff.cstore.transaction.commit" );
	my $resp = $req->recv();

	if(!$resp) {
		throw OpenSRF::EX::ERROR ("Unable to commit db session");
	}

	if(UNIVERSAL::isa($resp,"Error")) { 
		throw $resp ($resp->stringify); 
	}

	if(!$resp->content) {
		throw OpenSRF::EX::ERROR ("Unable to commit db session");
	}

	$session->finish();
	$session->disconnect();
	$session->kill_me();
}

sub rollback_db_session {
	my( $self, $session ) = @_;

	my $req = $session->request("sharestuff.cstore.transaction.rollback");
	my $resp = $req->recv();
	if(UNIVERSAL::isa($resp,"Error")) { throw $resp;  }

	$session->finish();
	$session->disconnect();
	$session->kill_me();
}


# returns undef it the event is not an ILS event
# returns the event code otherwise
sub event_code {
	my( $self, $evt ) = @_;
	return $evt->{ilsevent} if $self->is_event($evt);
	return undef;
}

# some events, in particular auto-generated events, don't have an 
# ilsevent key.  treat hashes with a 'textcode' key as events.
sub is_event {
	my ($self, $evt) = @_;
	return (
		ref($evt) eq 'HASH' and (
			defined $evt->{ilsevent} or
			defined $evt->{textcode}
		)
	);
}

# ---------------------------------------------------------------------------
# Checks to see if a user is logged in.  Returns the user record on success,
# throws an exception on error.
# ---------------------------------------------------------------------------
sub check_user_session {
	my( $self, $user_session ) = @_;

	my $content = $self->simplereq( 
		'sharestuff.auth', 
		'sharestuff.auth.session.retrieve', $user_session);

    return undef if (!$content) or $self->event_code($content);
	return $content;
}

# generic simple request returning a scalar value
sub simplereq {
	my($self, $service, $method, @params) = @_;
	return $self->simple_scalar_request($service, $method, @params);
}


sub simple_scalar_request {
	my($self, $service, $method, @params) = @_;

	my $session = OpenSRF::AppSession->create( $service );

	my $request = $session->request( $method, @params );

	my $val;
	my $err;
	try  {

		$val = $request->gather(1);	

	} catch Error with {
		$err = shift;
	};

	if( $err ) {
		warn "received error : service=$service : method=$method : params=".Dumper(\@params) . "\n $err";
		throw $err ("Call to $service for method $method \n failed with exception: $err : " );
	}

	return $val;
}

sub build_org_tree {
	my( $self, $orglist ) = @_;

	return $orglist unless ref $orglist; 
    return $$orglist[0] if @$orglist == 1;

	my @list = sort { 
		$a->ou_type <=> $b->ou_type ||
		$a->name cmp $b->name } @$orglist;

	for my $org (@list) {

		next unless ($org);
        next if (!defined($org->parent_ou) || $org->parent_ou eq "");

		my ($parent) = grep { $_->id == $org->parent_ou } @list;
		next unless $parent;
		$parent->children([]) unless defined($parent->children); 
		push( @{$parent->children}, $org );
	}

	return $list[0];
}

sub fetch_user {
	my( $self, $userid ) = @_;
	my( $user, $evt );
	
	$logger->debug("Fetching user $userid from cstore");

	$user = $self->simplereq(
		'sharestuff.cstore',
		'sharestuff.cstore.direct.actor.user.retrieve', $userid );

	if(!$user) {
		$logger->info("User $userid not found in the db");
		$evt = ShareStuff::Event->new('ACTOR_USER_NOT_FOUND');
	}

	return ($user, $evt);
}

sub checkses {
	my( $self, $session ) = @_;
	my $user = $self->check_user_session($session) or 
        return (undef, ShareStuff::Event->new('NO_SESSION'));
    return ($user);
}


# verifiese the session and checks the permissions agains the
# session user and the user's home_ou as the org id
sub checksesperm {
	my( $self, $session, @perms ) = @_;
	my $user; my $evt; my $e; 
	$logger->debug("Checking user session $session and perms @perms");
	($user, $evt) = $self->checkses($session);
	return (undef, $evt) if $evt;
	$evt = $self->check_perms($user->id, $user->home_ou, @perms);
	return ($user, $evt);
}


sub checkrequestor {
	my( $self, $staffobj, $userid, @perms ) = @_;
	my $user; my $evt;
	$userid = $staffobj->id unless defined $userid;

	$logger->debug("checkrequestor(): requestor => " . $staffobj->id . ", target => $userid");

	if( $userid ne $staffobj->id ) {
		($user, $evt) = $self->fetch_user($userid);
		return (undef, $evt) if $evt;
		$evt = $self->check_perms( $staffobj->id, $user->home_ou, @perms );

	} else {
		$user = $staffobj;
	}

	return ($user, $evt);
}

sub checkses_requestor {
	my( $self, $authtoken, $targetid, @perms ) = @_;
	my( $requestor, $target, $evt );

	($requestor, $evt) = $self->checkses($authtoken);
	return (undef, undef, $evt) if $evt;

	($target, $evt) = $self->checkrequestor( $requestor, $targetid, @perms );
	return( $requestor, $target, $evt);
}

sub fetch_patron_standings {
	my $self = shift;
	$logger->debug("Fetching patron standings");	
	return $self->simplereq(
		'sharestuff.cstore', 
		'sharestuff.cstore.direct.config.standing.search.atomic', { id => { '!=' => undef } });
}


sub fetch_permission_group_tree {
	my $self = shift;
	$logger->debug("Fetching patron profiles");	
	return $self->simplereq(
		'sharestuff.actor', 
		'sharestuff.actor.groups.tree.retrieve' );
}

sub fetch_permission_group_descendants {
    my( $self, $profile ) = @_;
    my $group_tree = $self->fetch_permission_group_tree();
    my $start_here;
    my @groups;

    # FIXME: okay, so it's not an org tree, but it is compatible
    $self->walk_org_tree($group_tree, sub {
        my $g = shift;
        if ($g->id == $profile) {
            $start_here = $g;
        }
    });

    $self->walk_org_tree($start_here, sub {
        my $g = shift;
        push(@groups,$g->id);
    });

    return \@groups;
}

my %ORG_CACHE; # - these rarely change, so cache them..
sub fetch_org_unit {
	my( $self, $id ) = @_;
	return undef unless $id;
	return $id if( ref($id) eq 'Fieldmapper::actor::org_unit' );
	return $ORG_CACHE{$id} if $ORG_CACHE{$id};
	$logger->debug("Fetching org unit $id");
	my $evt = undef;

	my $org = $self->simplereq(
		'sharestuff.cstore', 
		'sharestuff.cstore.direct.actor.org_unit.retrieve', $id );
	$evt = ShareStuff::Event->new( 'ACTOR_ORG_UNIT_NOT_FOUND', id => $id ) unless $org;
	$ORG_CACHE{$id}  = $org;

	return ($org, $evt);
}

sub fetch_stat_cat {
	my( $self, $type, $id ) = @_;
	my( $cat, $evt );
	$logger->debug("Fetching $type stat cat: $id");
	$cat = $self->simplereq(
		'sharestuff.cstore', 
		"sharestuff.cstore.direct.$type.stat_cat.retrieve", $id );

	my $e = 'ASSET_STAT_CAT_NOT_FOUND';
	$e = 'ACTOR_STAT_CAT_NOT_FOUND' if $type eq 'actor';

	$evt = ShareStuff::Event->new( $e, id => $id ) unless $cat;
	return ( $cat, $evt );
}

sub fetch_stat_cat_entry {
	my( $self, $type, $id ) = @_;
	my( $entry, $evt );
	$logger->debug("Fetching $type stat cat entry: $id");
	$entry = $self->simplereq(
		'sharestuff.cstore', 
		"sharestuff.cstore.direct.$type.stat_cat_entry.retrieve", $id );

	my $e = 'ASSET_STAT_CAT_ENTRY_NOT_FOUND';
	$e = 'ACTOR_STAT_CAT_ENTRY_NOT_FOUND' if $type eq 'actor';

	$evt = ShareStuff::Event->new( $e, id => $id ) unless $entry;
	return ( $entry, $evt );
}

sub fetch_stat_cat_entry_default {
    my( $self, $type, $id ) = @_;
    my( $entry_default, $evt );
    $logger->debug("Fetching $type stat cat entry default: $id");
    $entry_default = $self->simplereq(
        'sharestuff.cstore', 
        "sharestuff.cstore.direct.$type.stat_cat_entry_default.retrieve", $id );

    my $e = 'ASSET_STAT_CAT_ENTRY_DEFAULT_NOT_FOUND';
    $e = 'ACTOR_STAT_CAT_ENTRY_DEFAULT_NOT_FOUND' if $type eq 'actor';

    $evt = ShareStuff::Event->new( $e, id => $id ) unless $entry_default;
    return ( $entry_default, $evt );
}

sub fetch_stat_cat_entry_default_by_stat_cat_and_org {
    my( $self, $type, $stat_cat, $orgId ) = @_;
    my $entry_default;
    $logger->info("### Fetching $type stat cat entry default with stat_cat $stat_cat owned by org_unit $orgId");
    $entry_default = $self->simplereq(
        'sharestuff.cstore', 
        "sharestuff.cstore.direct.$type.stat_cat_entry_default.search.atomic", 
        { stat_cat => $stat_cat, owner => $orgId } );

    $entry_default = $entry_default->[0];
    return ($entry_default, undef) if $entry_default;

    my $e = 'ASSET_STAT_CAT_ENTRY_DEFAULT_NOT_FOUND';
    $e = 'ACTOR_STAT_CAT_ENTRY_DEFAULT_NOT_FOUND' if $type eq 'actor';
    return (undef, ShareStuff::Event->new($e) );
}

sub find_org {
	my( $self, $org_tree, $orgid )  = @_;
    return undef unless $org_tree and defined $orgid;
	return $org_tree if ( $org_tree->id eq $orgid );
	return undef unless ref($org_tree->children);
	for my $c (@{$org_tree->children}) {
		my $o = $self->find_org($c, $orgid);
		return $o if $o;
	}
	return undef;
}

sub DB_UPDATE_FAILED { 
	my( $self, $payload ) = @_;
	return ShareStuff::Event->new('DATABASE_UPDATE_FAILED', 
		payload => ($payload) ? $payload : undef ); 
}

sub storagereq {
	my( $self, $method, @params ) = @_;
	return $self->simplereq(
		'sharestuff.cstore', $method, @params );
}

sub storagereq_xact {
	my($self, $method, @params) = @_;
	my $ses = $self->start_db_session();
	my $val = $ses->request($method, @params)->gather(1);
	$self->rollback_db_session($ses);
    return $val;
}

sub cstorereq {
	my( $self, $method, @params ) = @_;
	return $self->simplereq(
		'sharestuff.cstore', $method, @params );
}

sub event_equals {
	my( $self, $e, $name ) =  @_;
	if( $e and ref($e) eq 'HASH' and 
		defined($e->{textcode}) and $e->{textcode} eq $name ) {
		return 1 ;
	}
	return 0;
}

sub logmark {
	my( undef, $f, $l ) = caller(0);
	my( undef, undef, undef, $s ) = caller(1);
	$s =~ s/.*:://g;
	$f =~ s/.*\///g;
	$logger->debug("LOGMARK: $f:$l:$s");
}

sub walk_org_tree {
	my( $self, $node, $callback ) = @_;
	return unless $node;
	$callback->($node);
	if( $node->children ) {
		$self->walk_org_tree($_, $callback) for @{$node->children};
	}
}

sub is_true {
	my( $self, $item ) = @_;
	return 1 if $item and $item !~ /^f$/i;
	return 0;
}


sub patientreq {
    my ($self, $client, $service, $method, @params) = @_;
    my ($response, $err);

    my $session = create OpenSRF::AppSession($service);
    my $request = $session->request($method, @params);

    my $spurt = 10;
    my $give_up = time + 1000;

    try {
        while (time < $give_up) {
            $response = $request->recv("timeout" => $spurt);
            last if $request->complete;

            $client->status(new OpenSRF::DomainObject::oilsContinueStatus);
        }
    } catch Error with {
        $err = shift;
    };

    if ($err) {
        warn "received error : service=$service : method=$method : params=".Dumper(\@params) . "\n $err";
        throw $err ("Call to $service for method $method \n failed with exception: $err : " );
    }

    return $response->content;
}

sub ou_ancestor_setting_value {
    my($self, $org_id, $name, $e) = @_;
    $e = $e || ShareStuff::CStoreEditor->new;
    my $set = $self->ou_ancestor_setting($org_id, $name, $e);
    return $set->{value} if $set;
    return undef;
}


# If an authentication token is provided AND this org unit setting has a
# view_perm, then make sure the user referenced by the auth token has
# that permission.  This means that if you call this method without an
# authtoken param, you can get whatever org unit setting values you want.
# API users beware.
#
# NOTE: If you supply an editor ($e) arg AND an auth token arg, the editor's
# authtoken is checked, but the $auth arg is NOT checked.  To say that another
# way, be sure NOT to pass an editor argument if you want your token checked.
# Otherwise the auth arg is just a flag saying "check the editor".  

sub ou_ancestor_setting {
    my( $self, $orgid, $name, $e, $auth ) = @_;
    $e = $e || ShareStuff::CStoreEditor->new(
        (defined $auth) ? (authtoken => $auth) : ()
    );
    my $coust = $e->retrieve_config_org_unit_setting_type([
        $name, {flesh => 1, flesh_fields => {coust => ['view_perm']}}
    ]);

    if ($auth && $coust && $coust->view_perm) {
        # And you can't have permission if you don't have a valid session.
        return undef if not $e->checkauth;
        # And now that we know you MIGHT have permission, we check it.
        return undef if not $e->allowed($coust->view_perm->code, $orgid);
    }

    my $query = {from => ['actor.org_unit_ancestor_setting', $name, $orgid]};
    my $setting = $e->json_query($query)->[0];
    return undef unless $setting;
    return {org => $setting->{org_unit}, value => OpenSRF::Utils::JSON->JSON2perl($setting->{value})};
}	
		

# returns the ISO8601 string representation of the requested epoch in GMT
sub epoch2ISO8601 {
    my( $self, $epoch ) = @_;
    my ($sec,$min,$hour,$mday,$mon,$year) = gmtime($epoch);
    $year += 1900; $mon += 1;
    my $date = sprintf(
        '%s-%0.2d-%0.2dT%0.2d:%0.2d:%0.2d-00',
        $year, $mon, $mday, $hour, $min, $sec);
    return $date;
}
			
sub find_highest_perm_org {
	my ( $self, $perm, $userid, $start_org, $org_tree ) = @_;
	my $org = $self->find_org($org_tree, $start_org );

	my $lastid = -1;
	while( $org ) {
		last if ($self->check_perms( $userid, $org->id, $perm )); # perm failed
		$lastid = $org->id;
		$org = $self->find_org( $org_tree, $org->parent_ou() );
	}

	return $lastid;
}


# returns the org_unit ID's 
sub user_has_work_perm_at {
    my($self, $e, $perm, $options, $user_id) = @_;
    $options ||= {};
    $user_id = (defined $user_id) ? $user_id : $e->requestor->id;

    my $func = 'permission.usr_has_perm_at';
    $func = $func.'_all' if $$options{descendants};

    my $orgs = $e->json_query({from => [$func, $user_id, $perm]});
    $orgs = [map { $_->{ (keys %$_)[0] } } @$orgs];

    return $orgs unless $$options{objects};

    return $e->search_actor_org_unit({id => $orgs});
}

sub get_user_work_ou_ids {
    my($self, $e, $userid) = @_;
    my $work_orgs = $e->json_query({
        select => {puwoum => ['work_ou']},
        from => 'puwoum',
        where => {usr => $e->requestor->id}});

    return [] unless @$work_orgs;
    my @work_orgs;
    push(@work_orgs, $_->{work_ou}) for @$work_orgs;

    return \@work_orgs;
}


my $org_types;
sub get_org_types {
	my($self, $client) = @_;
	return $org_types if $org_types;
	return $org_types = ShareStuff::CStoreEditor->new->retrieve_all_actor_org_unit_type();
}

my %ORG_TREE;
sub get_org_tree {
	my $self = shift;
	my $locale = shift || '';
	my $cache = OpenSRF::Utils::Cache->new("global", 0);
	my $tree = $ORG_TREE{$locale} || $cache->get_cache("orgtree.$locale");
	return $tree if $tree;

	my $ses = ShareStuff::CStoreEditor->new;
	$ses->session->session_locale($locale);
	$tree = $ses->search_actor_org_unit( 
		[
			{"parent_ou" => undef },
			{
				flesh				=> -1,
				flesh_fields	=> { aou =>  ['children'] },
				order_by			=> { aou => 'name'}
			}
		]
	)->[0];

    $ORG_TREE{$locale} = $tree;
	$cache->put_cache("orgtree.$locale", $tree);
	return $tree;
}

sub get_org_descendants {
	my($self, $org_id, $depth) = @_;

	my $select = {
		transform => 'actor.org_unit_descendants',
		column => 'id',
		result_field => 'id',
	};
	$select->{params} = [$depth] if defined $depth;

	my $org_list = ShareStuff::CStoreEditor->new->json_query({
		select => {aou => [$select]},
        from => 'aou',
		where => {id => $org_id}
	});
	my @orgs;
	push(@orgs, $_->{id}) for @$org_list;
	return \@orgs;
}

sub get_org_ancestors {
	my($self, $org_id, $use_cache) = @_;

    my ($cache, $orgs);

    if ($use_cache) {
        $cache = OpenSRF::Utils::Cache->new("global", 0);
        $orgs = $cache->get_cache("org.ancestors.$org_id");
        return $orgs if $orgs;
    }

	my $org_list = ShareStuff::CStoreEditor->new->json_query({
		select => {
			aou => [{
				transform => 'actor.org_unit_ancestors',
				column => 'id',
				result_field => 'id',
				params => []
			}],
		},
		from => 'aou',
		where => {id => $org_id}
	});

	$orgs = [ map { $_->{id} } @$org_list ];

    $cache->put_cache("org.ancestors.$org_id", $orgs) if $use_cache;
	return $orgs;
}

sub get_org_full_path {
	my($self, $org_id, $depth) = @_;

    my $query = {
        select => {
			aou => [{
				transform => 'actor.org_unit_full_path',
				column => 'id',
				result_field => 'id',
			}],
		},
		from => 'aou',
		where => {id => $org_id}
	};

    $query->{select}->{aou}->[0]->{params} = [$depth] if defined $depth;
	my $org_list = ShareStuff::CStoreEditor->new->json_query($query);
    return [ map {$_->{id}} @$org_list ];
}

# returns the ID of the org unit ancestor at the specified depth
sub org_unit_ancestor_at_depth {
    my($class, $org_id, $depth) = @_;
    my $resp = ShareStuff::CStoreEditor->new->json_query(
        {from => ['actor.org_unit_ancestor_at_depth', $org_id, $depth]})->[0];
    return ($resp) ? $resp->{id} : undef;
}

# returns the user's configured locale as a string.  Defaults to en-US if none is configured.
sub get_user_locale {
	my($self, $user_id, $e) = @_;
	$e ||= ShareStuff::CStoreEditor->new;

	# first, see if the user has an explicit locale set
	my $setting = $e->search_actor_user_setting(
		{usr => $user_id, name => 'global.locale'})->[0];
	return OpenSRF::Utils::JSON->JSON2perl($setting->value) if $setting;

	my $user = $e->retrieve_actor_user($user_id) or return $e->event;
	return $self->get_org_locale($user->home_ou, $e);
}

# returns org locale setting
sub get_org_locale {
	my($self, $org_id, $e) = @_;
	$e ||= ShareStuff::CStoreEditor->new;

	my $locale;
	if(defined $org_id) {
		$locale = $self->ou_ancestor_setting_value($org_id, 'global.default_locale', $e);
		return $locale if $locale;
	}

	# system-wide default
	my $sclient = OpenSRF::Utils::SettingsClient->new;
	$locale = $sclient->config_value('default_locale');
    return $locale if $locale;

	# if nothing else, fallback to locale=cowboy
	return 'en-US';
}


# xml-escape non-ascii characters
sub entityize { 
    my($self, $string, $form) = @_;
	$form ||= "";

	# If we're going to convert non-ASCII characters to XML entities,
	# we had better be dealing with a UTF8 string to begin with
	$string = decode_utf8($string);

	if ($form eq 'D') {
		$string = NFD($string);
	} else {
		$string = NFC($string);
	}

	# Convert raw ampersands to entities
	$string =~ s/&(?!\S+;)/&amp;/gso;

	# Convert Unicode characters to entities
	$string =~ s/([\x{0080}-\x{fffd}])/sprintf('&#x%X;',ord($1))/sgoe;

	return $string;
}

# x0000-x0008 isn't legal in XML documents
# XXX Perhaps this should just go into our standard entityize method
sub strip_ctrl_chars {
	my ($self, $string) = @_;

	$string =~ s/([\x{0000}-\x{0008}])//sgoe; 
	return $string;
}

sub create_uuid_string {
    return create_UUID_as_string();
}

# returns undef on success, Event on error
sub log_user_activity {
    my ($class, $user_id, $who, $what, $e, $async) = @_;

    my $commit = 0;
    if (!$e) {
        $e = ShareStuff::CStoreEditor->new(xact => 1);
        $commit = 1;
    }

    my $res = $e->json_query({
        from => [
            'actor.insert_usr_activity', 
            $user_id, $who, $what, OpenSRF::AppSession->ingress
        ]
    });

    if ($res) { # call returned OK

        $e->commit   if $commit and @$res;
        $e->rollback if $commit and !@$res;

    } else {
        return $e->die_event;
    }

    return undef;
}

# Compare two dates, date1 and date2. If date2 is not defined, then
# DateTime->now will be used. Assumes dates are in ISO8601 format as
# supported by DateTime::Format::ISO8601. (A future enhancement might
# be to support other formats.)
#
# Returns -1 if $date1 < $date2
# Returns 0 if $date1 == $date2
# Returns 1 if $date1 > $date2
sub datecmp {
    my $self = shift;
    my $date1 = shift;
    my $date2 = shift;

    # Check for timezone offsets and limit them to 2 digits:
    if ($date1 && $date1 =~ /(?:-|\+)\d\d\d\d$/) {
        $date1 = substr($date1, 0, length($date1) - 2);
    }
    if ($date2 && $date2 =~ /(?:-|\+)\d\d\d\d$/) {
        $date2 = substr($date2, 0, length($date2) - 2);
    }

    # check date1:
    unless (UNIVERSAL::isa($date1, "DateTime")) {
        $date1 = DateTime::Format::ISO8601->parse_datetime($date1);
    }

    # Check for date2:
    unless ($date2) {
        $date2 = DateTime->now;
    } else {
        unless (UNIVERSAL::isa($date2, "DateTime")) {
            $date2 = DateTime::Format::ISO8601->parse_datetime($date2);
        }
    }

    return DateTime->compare($date1, $date2);
}

1;

